@@ -1,16 +1,25 @@
-import astropy.io.fits as pyfits
-import numpy as np
-from soxs.utils import parse_prng, parse_value, \
-    ensure_numpy_array, mylog, process_fits_string
-from soxs.spatial import construct_wcs
-from astropy.units import Quantity
-from collections.abc import Sequence
 import os
+from collections.abc import Sequence
+
+import numpy as np
+from astropy.io import fits
+from astropy.units import Quantity
 from pathlib import Path
 
+from soxs.spatial import construct_wcs
+from soxs.utils import (
+    ensure_numpy_array,
+    mylog,
+    parse_prng,
+    parse_value,
+    process_fits_string,
+    soxs_cfg,
+)
+
 
 def _parse_catalog_entry(src):
     import re
+
     brackets = re.findall(r"[^[]*\[([^]]*)\]", src)
     src_file = src.split("[")[0]
     ext = brackets[0].lower().split(",")
@@ -43,8 +52,8 @@ class LazyReadSimputCatalog(Sequence):
 
 def read_simput_catalog(simput_file):
     r"""
-    Read events from a SIMPUT catalog. This will read 
-    all of the sources in the catalog.
+    Read events from a SIMPUT catalog. This will read
+    all the sources in the catalog.
 
     Parameters
     ----------
@@ -53,22 +62,24 @@ def read_simput_catalog(simput_file):
 
     Returns
     -------
-    1. Lists of dicts of NumPy arrays of the positions 
+    1. Lists of dicts of NumPy arrays of the positions
        and energies of the events from the sources.
     2. NumPy arrays of the parameters of the sources.
     """
     sc = SimputCatalog.from_file(simput_file)
-    parameters = {"emin": sc.emin,
-                  "emax": sc.emax,
-                  "src_names": sc.src_names,
-                  "flux": sc.fluxes}
+    parameters = {
+        "emin": sc.emin,
+        "emax": sc.emax,
+        "src_names": sc.src_names,
+        "flux": sc.fluxes,
+    }
     return LazyReadSimputCatalog(sc), parameters
 
 
 class SimputCatalog:
-
-    def __init__(self, spectra, images, src_names, ra, dec, fluxes,
-                 emin, emax, filename):
+    def __init__(
+        self, spectra, images, src_names, ra, dec, fluxes, emin, emax, filename
+    ):
         self.spectra = ensure_numpy_array(spectra)
         self.images = ensure_numpy_array(images)
         self.src_names = ensure_numpy_array(src_names)
@@ -80,11 +91,10 @@ class SimputCatalog:
         self.filename = filename
         self.num_sources = self.spectra.size
         # timing not yet supported
-        self.timing = np.array(["NULL"]*self.num_sources)
+        self.timing = np.array(["NULL"] * self.num_sources)
 
     @classmethod
-    def from_source(cls, filename, source, src_filename=None,
-                    overwrite=False):
+    def from_source(cls, filename, source, src_filename=None, overwrite=False):
         """
         Create a new :class:`~soxs.simput.SimputCatalog`
         instance using a single :class:`~soxs.simput.SimputSource`
@@ -94,7 +104,7 @@ class SimputCatalog:
         ----------
         filename : string
             The name of the SIMPUT catalog file to write. The
-            file will be written 
+            file will be written
         source : :class:`~soxs.simput.SimputSource`
             The SIMPUT source to create the catalog with.
         src_filename : string, optional
@@ -102,7 +112,7 @@ class SimputCatalog:
             to. By default, the source will be written to the same
             file as the SIMPUT catalog.
         overwrite : boolean, optional
-            Whether or not to overwrite an existing file with
+            Whether to overwrite an existing file with
             the same name. If src_filename=None and the source is
             to the written to the SIMPUT catalog file, then this
             argument is ignored. If src_filename is another value,
@@ -114,8 +124,7 @@ class SimputCatalog:
         if os.path.exists(filename) and not overwrite:
             raise IOError(f"{filename} exists and overwrite=False!")
         sc._write_catalog(overwrite=overwrite)
-        sc.append(source, src_filename=src_filename,
-                  overwrite=overwrite)
+        sc.append(source, src_filename=src_filename, overwrite=overwrite)
         return sc
 
     @classmethod
@@ -130,7 +139,7 @@ class SimputCatalog:
             The name of the SIMPUT catalog file to read the
             catalog and photon lists from.
         """
-        f_simput = pyfits.open(filename)
+        f_simput = fits.open(filename)
         fluxes = f_simput["src_cat"].data["flux"]
         src_names = f_simput["src_cat"].data["src_name"]
         e_min = f_simput["src_cat"].data["e_min"]
@@ -141,19 +150,20 @@ class SimputCatalog:
         if "IMAGE" in f_simput["src_cat"].columns.names:
             images = f_simput["src_cat"].data["image"]
         else:
-            images = ["NULL"]*len(spectra)
+            images = ["NULL"] * len(spectra)
         f_simput.close()
 
-        return cls(spectra, images, src_names, ra, dec, fluxes, e_min, e_max,
-                   filename)
+        return cls(spectra, images, src_names, ra, dec, fluxes, e_min, e_max, filename)
 
     def read_source(self, spec):
         """
         Read a source from the SIMPUT catalog with the identifier
         *spec* for the SPECTRUM field.
         """
-        from .spectra import Spectrum
         from astropy.io.fits.column import _VLF
+
+        from .spectra import Spectrum
+
         i = np.where(self.spectra == spec)[0][0]
         src_file, extname, extver, row = _parse_catalog_entry(spec)
         # If no file is specified, assume the catalog and
@@ -163,13 +173,14 @@ class SimputCatalog:
         else:
             fn_src = src_file
         ext = extname if extver is None else (extname, extver)
-        data = pyfits.getdata(fn_src, ext)
+        data = fits.getdata(fn_src, ext)
         if extname == "phlist":
             ra = Quantity(data["ra"], "deg")
             dec = Quantity(data["dec"], "deg")
             energy = Quantity(data["energy"], "keV")
-            src = SimputPhotonList(ra, dec, energy, self.fluxes[i],
-                                   name=self.src_names[i])
+            src = SimputPhotonList(
+                ra, dec, energy, self.fluxes[i], name=self.src_names[i]
+            )
         elif extname == "spectrum":
             emid = data["energy"]
             flux = data["fluxdensity"]
@@ -181,12 +192,12 @@ class SimputCatalog:
                     row = np.where(data["name"])[0][0]
                 emid = emid[row, :]
                 flux = flux[row, :]
+            # This assumes linear binning for now!!!
             de = np.diff(emid)[0]
             ebins = np.append(emid - 0.5 * de, emid[-1] + 0.5 * de)
-            spec = Spectrum(ebins, flux)
+            spec = Spectrum(ebins, flux, binscale="linear")
             if self.images[i].upper() != "NULL":
-                src_file, extname, extver, _ = \
-                    _parse_catalog_entry(self.images[i])
+                src_file, extname, extver, _ = _parse_catalog_entry(self.images[i])
                 # If no file is specified, assume the catalog and
                 # source are in the same file
                 if src_file == "":
@@ -194,14 +205,21 @@ class SimputCatalog:
                 else:
                     fn_img = src_file
                 ext = extname if extver is None else (extname, extver)
-                imdata, imheader = pyfits.getdata(
-                    fn_img, ext, header=True)
-                imhdu = pyfits.ImageHDU(data=imdata,
-                                        header=imheader)
+                imdata, imheader = fits.getdata(fn_img, ext, header=True)
+                imhdu = fits.ImageHDU(data=imdata, header=imheader)
             else:
                 imhdu = None
-            src = SimputSpectrum(spec, self.ra[i], self.dec[i],
-                                 name=self.src_names[i], imhdu=imhdu)
+            src = SimputSpectrum(
+                self.emin[i],
+                self.emax[i],
+                self.fluxes[i],
+                emid,
+                flux,
+                self.ra[i],
+                self.dec[i],
+                name=self.src_names[i],
+                imhdu=imhdu,
+            )
         else:
             raise RuntimeError
 
@@ -212,22 +230,22 @@ class SimputCatalog:
         Write the SIMPUT catalog to disk.
         """
         src_id = np.arange(self.num_sources)
-        col1 = pyfits.Column(name='SRC_ID', format='J', array=src_id)
-        col2 = pyfits.Column(name='RA', format='D', array=self.ra)
-        col3 = pyfits.Column(name='DEC', format='D', array=self.dec)
-        col4 = pyfits.Column(name='E_MIN', format='D', array=self.emin)
-        col5 = pyfits.Column(name='E_MAX', format='D', array=self.emax)
-        col6 = pyfits.Column(name='FLUX', format='D', array=self.fluxes)
-        col7 = pyfits.Column(name='SPECTRUM', format='80A', array=self.spectra)
-        col8 = pyfits.Column(name='IMAGE', format='80A', array=self.images)
-        col9 = pyfits.Column(name='TIMING', format='80A', array=self.timing)
-        col10 = pyfits.Column(name='SRC_NAME', format='80A',
-                              array=self.src_names)
+        col1 = fits.Column(name="SRC_ID", format="J", array=src_id)
+        col2 = fits.Column(name="RA", format="D", array=self.ra)
+        col3 = fits.Column(name="DEC", format="D", array=self.dec)
+        col4 = fits.Column(name="E_MIN", format="D", array=self.emin)
+        col5 = fits.Column(name="E_MAX", format="D", array=self.emax)
+        col6 = fits.Column(name="FLUX", format="D", array=self.fluxes)
+        col7 = fits.Column(name="SPECTRUM", format="80A", array=self.spectra)
+        col8 = fits.Column(name="IMAGE", format="80A", array=self.images)
+        col9 = fits.Column(name="TIMING", format="80A", array=self.timing)
+        col10 = fits.Column(name="SRC_NAME", format="80A", array=self.src_names)
 
-        coldefs = pyfits.ColDefs([col1, col2, col3, col4, col5,
-                                  col6, col7, col8, col9, col10])
+        coldefs = fits.ColDefs(
+            [col1, col2, col3, col4, col5, col6, col7, col8, col9, col10]
+        )
 
-        wrhdu = pyfits.BinTableHDU.from_columns(coldefs)
+        wrhdu = fits.BinTableHDU.from_columns(coldefs)
         wrhdu.name = "SRC_CAT"
 
         wrhdu.header["HDUCLASS"] = "HEASARC"
@@ -243,12 +261,12 @@ class SimputCatalog:
         wrhdu.header["TUNIT6"] = "erg/s/cm**2"
 
         if os.path.exists(self.filename) and not overwrite:
-            with pyfits.open(self.filename, mode='update') as f:
+            with fits.open(self.filename, mode="update") as f:
                 f["SRC_CAT"] = wrhdu
                 f.flush()
         else:
-            f = [pyfits.PrimaryHDU(), wrhdu]
-            pyfits.HDUList(f).writeto(self.filename, overwrite=True)
+            f = [fits.PrimaryHDU(), wrhdu]
+            fits.HDUList(f).writeto(self.filename, overwrite=True)
 
     def append(self, source, src_filename=None, overwrite=False):
         """
@@ -263,7 +281,7 @@ class SimputCatalog:
             to. By default, the source will be written to the same
             file as the SIMPUT catalog.
         overwrite : boolean, optional
-            Whether or not to overwrite an existing file with
+            Whether to overwrite an existing file with
             the same name. If src_filename=None and the source is
             to be written to the SIMPUT catalog file, then this
             argument is ignored. If src_filename is another value,
@@ -285,15 +303,15 @@ class SimputCatalog:
             # Don't overwrite the SIMPUT catalog file!!
             overwrite = False
         elif overwrite and os.path.exists(src_filename):
-            mylog.warning(f"Overwriting {src_filename}.")
+            mylog.warning("Overwriting %s.", src_filename)
             os.remove(src_filename)
 
         extver = _determine_extver(src_filename, source.src_type.upper())
         if src_filename != self.filename:
             src_fn = os.path.join(
-                os.path.relpath(Path(src_filename).parent,
-                                Path(self.filename).parent),
-                os.path.basename(src_filename))
+                os.path.relpath(Path(src_filename).parent, Path(self.filename).parent),
+                os.path.basename(src_filename),
+            )
         else:
             src_fn = ""
         spec = f"{src_fn}[{source.src_type.upper()},{extver}]"
@@ -310,17 +328,18 @@ class SimputCatalog:
         self.images = np.append(self.images, img)
 
         self._write_catalog()
-        source._write_source(src_filename, extver, img_extver=img_extver,
-                             overwrite=overwrite)
+        source._write_source(
+            src_filename, extver, img_extver=img_extver, overwrite=overwrite
+        )
 
 
 def _determine_extver(fn, extname):
     extver = 1
     if os.path.exists(fn):
-        with pyfits.open(fn) as f:
+        with fits.open(fn) as f:
             for hdu in f:
                 if hdu.name == extname:
-                    extver = hdu.header["EXTVER"]+1
+                    extver = hdu.header["EXTVER"] + 1
     return extver
 
 
@@ -344,7 +363,7 @@ class SimputSource:
     def _write_source(self, filename, extver, img_extver=None, overwrite=False):
         coldefs, header = self._get_source_hdu()
 
-        tbhdu = pyfits.BinTableHDU.from_columns(coldefs)
+        tbhdu = fits.BinTableHDU.from_columns(coldefs)
         tbhdu.name = self.src_type.upper()
 
         if self.src_type == "phlist":
@@ -361,21 +380,21 @@ class SimputSource:
             self.imhdu.header["EXTVER"] = img_extver
 
         if os.path.exists(filename) and not overwrite:
-            mylog.info(f"Appending this source to {filename}.")
-            with pyfits.open(filename, mode='append') as f:
+            mylog.info("Appending source '%s' to %s.", self.name, filename)
+            with fits.open(filename, mode="append") as f:
                 f.append(tbhdu)
                 if self.imhdu is not None:
                     f.append(self.imhdu)
                 f.flush()
         else:
             if os.path.exists(filename):
-                mylog.warning(f"Overwriting {filename} with this source.")
+                mylog.warning("Overwriting %s with source '%s'.", filename, self.name)
             else:
-                mylog.info(f"Writing source to {filename}.")
-            f = [pyfits.PrimaryHDU(), tbhdu]
+                mylog.info("Writing source '%s' to %s.", self.name, filename)
+            f = [fits.PrimaryHDU(), tbhdu]
             if self.imhdu is not None:
                 f.append(self.imhdu)
-            pyfits.HDUList(f).writeto(filename, overwrite=overwrite)
+            fits.HDUList(f).writeto(filename, overwrite=overwrite)
 
         if self.imhdu is not None:
             self.imhdu.header["EXTVER"] = 1
@@ -384,26 +403,34 @@ class SimputSource:
 class SimputSpectrum(SimputSource):
     src_type = "spectrum"
 
-    def __init__(self, spec, ra, dec, name=None, imhdu=None):
-        super(SimputSpectrum, self).__init__(spec.ebins.value.min(),
-                                             spec.ebins.value.max(),
-                                             spec.total_flux.value, ra,
-                                             dec, name=name, imhdu=imhdu)
-        self.spec = spec
+    def __init__(
+        self, emin, emax, flux, energy, fluxdensity, ra, dec, name=None, imhdu=None
+    ):
+        super(SimputSpectrum, self).__init__(
+            emin, emax, flux, ra, dec, name=name, imhdu=imhdu
+        )
+        self.energy = energy
+        self.fluxdensity = fluxdensity
 
     def _get_source_hdu(self):
-        col1 = pyfits.Column(name='ENERGY', format='E',
-                             array=self.spec.emid.value)
-        col2 = pyfits.Column(name='FLUXDENSITY', format='D',
-                             array=self.spec.flux.value)
-        cols = [col1, col2]
+        col1 = fits.Column(
+            name="ENERGY",
+            unit="keV",
+            format="1PE()",
+            array=np.array([self.energy], dtype=np.object_),
+        )
+        col2 = fits.Column(
+            name="FLUXDENSITY",
+            unit="photons/s/cm**2/keV",
+            format="1PE()",
+            array=np.array([self.fluxdensity], dtype=np.object_),
+        )
+        col3 = fits.Column(name="NAME", format="48A", array=np.array([""]))
+        cols = [col1, col2, col3]
 
-        coldefs = pyfits.ColDefs(cols)
+        coldefs = fits.ColDefs(cols)
 
-        header = {"REFRA": self.ra,
-                  "REFDEC": self.dec,
-                  "TUNIT1": "keV",
-                  "TUNIT2": "photon/(cm**2*s*keV)"}
+        header = {"REFRA": self.ra, "REFDEC": self.dec}
 
         return coldefs, header
 
@@ -426,17 +453,26 @@ class SimputSpectrum(SimputSource):
             The Dec of the source in degrees.
         imhdu : string or :class:`~astropy.io.fits.ImageHDU` instance
             An image to be used to simulate an extended source. Can be an
-            ImageHDU instance or the name of a to read one from. If the
+            ImageHDU instance or the name of a file to read one from. If the
             name contains an HDU extension, e.g. "cluster.fits[1]" or
             "cluster.fits['perseus']", that extension will be loaded.
         """
         if isinstance(imhdu, str):
             imhdu = process_fits_string(imhdu)
-        return cls(spectral_model, ra, dec, name=name, imhdu=imhdu)
+        return cls(
+            spectral_model.ebins.value.min(),
+            spectral_model.ebins.value.max(),
+            spectral_model.total_energy_flux.value,
+            spectral_model.emid.value,
+            spectral_model.flux.value,
+            ra,
+            dec,
+            name=name,
+            imhdu=imhdu,
+        )
 
     @classmethod
-    def from_models(cls, name, spectral_model, spatial_model,
-                    width, nx):
+    def from_models(cls, name, spectral_model, spatial_model, width, nx):
         """
         Generate a SIMPUT spectrum from a spectral and a spatial
         model, and parameters for an image.
@@ -456,8 +492,9 @@ class SimputSpectrum(SimputSource):
             on a side.
         """
         imhdu = spatial_model.generate_image(width, nx)
-        return cls(spectral_model, spatial_model.ra0,
-                   spatial_model.dec0, name=name, imhdu=imhdu)
+        return cls.from_spectrum(
+            name, spectral_model, spatial_model.ra0, spatial_model.dec0, imhdu=imhdu
+        )
 
 
 class SimputPhotonList(SimputSource):
@@ -466,8 +503,7 @@ class SimputPhotonList(SimputSource):
     def __init__(self, ra, dec, energy, flux, name=None):
         emin = np.asarray(energy).min()
         emax = np.asarray(energy).max()
-        super(SimputPhotonList, self).__init__(
-            emin, emax, flux, 0.0, 0.0, name=name)
+        super(SimputPhotonList, self).__init__(emin, emax, flux, 0.0, 0.0, name=name)
         self.events = {"ra": ra, "dec": dec, "energy": energy}
         self.num_events = energy.size
 
@@ -482,11 +518,10 @@ class SimputPhotonList(SimputSource):
             yield key
 
     @classmethod
-    def from_models(cls, name, spectral_model, spatial_model,
-                    t_exp, area, prng=None):
+    def from_models(cls, name, spectral_model, spatial_model, t_exp, area, prng=None):
         """
         Generate a SIMPUT photon list from a spectral and a spatial
-        model. 
+        model.
 
         Parameters
         ----------
@@ -499,15 +534,15 @@ class SimputPhotonList(SimputSource):
         t_exp : float, (value, unit) tuple, or :class:`~astropy.units.Quantity`
             The exposure time in seconds.
         area : float, (value, unit) tuple, or :class:`~astropy.units.Quantity`
-            The effective area in cm**2. If one is creating 
-            events for a SIMPUT file, a constant should be 
-            used and it must be large enough so that a 
+            The effective area in cm**2. If one is creating
+            events for a SIMPUT file, a constant should be
+            used and it must be large enough so that a
             sufficiently large sample is drawn for the ARF.
         prng : :class:`~numpy.random.RandomState` object, integer, or None
-            A pseudo-random number generator. Typically will only 
-            be specified if you have a reason to generate the same 
-            set of random numbers, such as for a test. Default is None, 
-            which sets the seed based on the system time. 
+            A pseudo-random number generator. Typically will only
+            be specified if you have a reason to generate the same
+            set of random numbers, such as for a test. Default is None,
+            which sets the seed based on the system time.
         """
         prng = parse_prng(prng)
         t_exp = parse_value(t_exp, "s")
@@ -517,31 +552,43 @@ class SimputPhotonList(SimputSource):
         return cls(ra, dec, e, e.flux.value, name=name)
 
     def _get_source_hdu(self):
-        col1 = pyfits.Column(name='ENERGY', format='E',
-                             array=np.asarray(self["energy"]))
-        col2 = pyfits.Column(name='RA', format='D',
-                             array=np.asarray(self["ra"]))
-        col3 = pyfits.Column(name='DEC', format='D',
-                             array=np.asarray(self["dec"]))
+        col1 = fits.Column(name="ENERGY", format="E", array=np.asarray(self["energy"]))
+        col2 = fits.Column(name="RA", format="D", array=np.asarray(self["ra"]))
+        col3 = fits.Column(name="DEC", format="D", array=np.asarray(self["dec"]))
         cols = [col1, col2, col3]
 
-        coldefs = pyfits.ColDefs(cols)
+        coldefs = fits.ColDefs(cols)
 
-        header = {"REFRA": 0.0,
-                  "REFDEC": 0.0,
-                  "TUNIT1": "keV",
-                  "TUNIT2": "deg",
-                  "TUNIT3": "deg"}
+        header = {
+            "REFRA": 0.0,
+            "REFDEC": 0.0,
+            "TUNIT1": "keV",
+            "TUNIT2": "deg",
+            "TUNIT3": "deg",
+        }
 
         return coldefs, header
 
-    def plot(self, center, width, s=None, c=None, marker=None, stride=1,
-             emin=None, emax=None, label=None, fontsize=18, fig=None, 
-             ax=None, **kwargs):
+    def plot(
+        self,
+        center,
+        width,
+        s=None,
+        c=None,
+        marker=None,
+        stride=1,
+        emin=None,
+        emax=None,
+        label=None,
+        fontsize=18,
+        fig=None,
+        ax=None,
+        **kwargs,
+    ):
         """
-        Plot event coordinates from this photon list in a scatter plot, 
+        Plot event coordinates from this photon list in a scatter plot,
         optionally restricting the photon energies which are plotted
-        and using only a subset of the photons. 
+        and using only a subset of the photons.
 
         Parameters
         ----------
@@ -576,6 +623,7 @@ class SimputPhotonList(SimputSource):
         """
         import matplotlib.pyplot as plt
         from astropy.visualization.wcsaxes import WCSAxes
+
         if fig is None:
             fig = plt.figure(figsize=(10, 10))
         if ax is None:
@@ -592,24 +640,26 @@ class SimputPhotonList(SimputSource):
             emax = self.emax
         else:
             emax = parse_value(emax, "keV")
-        idxs = np.logical_and(self["energy"].value >= emin, 
-                              self["energy"].value <= emax)
+        idxs = np.logical_and(
+            self["energy"].value >= emin, self["energy"].value <= emax
+        )
         ra = self["ra"][idxs][::stride].value
         dec = self["dec"][idxs][::stride].value
         x, y = wcs.wcs_world2pix(ra, dec, 1)
         ax.scatter(x, y, s=s, c=c, marker=marker, label=label, **kwargs)
         x0, y0 = wcs.wcs_world2pix(center[0], center[1], 1)
-        width = parse_value(width, "arcmin")*60.0
-        ax.set_xlim(x0-0.5*width, x0+0.5*width)
-        ax.set_ylim(y0-0.5*width, y0+0.5*width)
+        width = parse_value(width, "arcmin") * 60.0
+        ax.set_xlim(x0 - 0.5 * width, x0 + 0.5 * width)
+        ax.set_ylim(y0 - 0.5 * width, y0 + 0.5 * width)
         ax.set_xlabel("RA")
         ax.set_ylabel("Dec")
-        ax.tick_params(axis='both', labelsize=fontsize)
+        ax.tick_params(axis="both", labelsize=fontsize)
         return fig, ax
 
 
-def write_photon_list(simput_prefix, phlist_prefix, flux, ra, dec, 
-                      energy, overwrite=False):
+def write_photon_list(
+    simput_prefix, phlist_prefix, flux, ra, dec, energy, overwrite=False
+):
     """
     This function is designed to preserve backwards-compatibility
     with pyXSIM 2.x. It will be removed in a future release.
@@ -617,6 +667,124 @@ def write_photon_list(simput_prefix, phlist_prefix, flux, ra, dec,
     simput_file = f"{simput_prefix}_simput.fits"
     phlist_file = f"{phlist_prefix}_phlist.fits"
     phlist = SimputPhotonList(ra, dec, energy, flux)
-    SimputCatalog.from_source(simput_file, phlist, 
-                              src_filename=phlist_file,
-                              overwrite=overwrite)
\ No newline at end of file
+    SimputCatalog.from_source(
+        simput_file, phlist, src_filename=phlist_file, overwrite=overwrite
+    )
+
+
+def make_bkgnd_simput(
+    filename,
+    exp_time,
+    area,
+    fov,
+    sky_center,
+    absorb_model=None,
+    nH=None,
+    frgnd_velocity=None,
+    frgnd_spec_model=None,
+    overwrite=False,
+    input_sources=None,
+    output_sources=None,
+    diffuse_unresolved=True,
+    prng=None,
+):
+    """
+    Create a SIMPUT catalog including the Galactic foreground and the
+    CXB point sources, suitable for including in a simulation by another
+    package that can take SIMPUT inputs, such as SIXTE, SIMX, or MARX.
+
+    Parameters
+    ----------
+    filename : string
+        The filename to write the SIMPUT catalog to.
+    exp_time : float, (value, unit) tuple, or :class:`~astropy.units.Quantity`
+        The exposure time in seconds.
+    area : float, (value, unit) tuple, or :class:`~astropy.units.Quantity`
+        The effective area in cm**2. If one is creating
+        events for a SIMPUT file, a constant should be
+        used and it must be large enough so that a
+        sufficiently large sample is drawn for the ARF.
+    fov : float, (value, unit) tuple, or :class:`~astropy.units.Quantity`
+        The field of view in arcminutes.
+    sky_center : array-like
+        The center RA, Dec of the field of view in degrees.
+    absorb_model : string, optional
+        The absorption model to use, "wabs" or "tbabs".
+        Defaults to the value in the SOXS configuration file.
+    nH : float, (value, unit) tuple, or :class:`~astropy.units.Quantity`, optional
+        The hydrogen column in units of 10**22 atoms/cm**2.
+        Defaults to the value in the SOXS configuration file.
+    frgnd_velocity : float, (value, unit) tuple, or :class:`~astropy.units.Quantity`, optional
+        The velocity dispersion to apply to the Galactic foreground lines.
+        Defaults to the value in the SOXS configuration file.
+    frgnd_spec_model : string, optional
+        The model for the Galactic foreground spectrum to use, currently
+        either "default" or "halosat". Defaults to the value in the SOXS
+        configuration file.
+    overwrite : boolean, optional
+        Set to True to overwrite previous files. Default: False
+    input_sources : string, optional
+        If set to a filename, input the source positions, fluxes,
+        and spectral indices from an ASCII table instead of generating
+        them. Default: None
+    output_sources : string, optional
+        If set to a filename, output the properties of the sources
+        within the field of view to a file. Default: None
+    diffuse_unresolved : boolean, optional
+        Add a diffuse component across the entire field of view to represent
+        the unresolved flux from sources at very small fluxes. Default: True
+    prng : :class:`~numpy.random.RandomState` object, integer, or None
+        A pseudo-random number generator. Typically will only
+        be specified if you have a reason to generate the same
+        set of random numbers, such as for a test. Default is None,
+        which sets the seed based on the system time.
+    """
+    from soxs.background.diffuse import _make_frgnd_spectrum
+    from soxs.background.point_sources import make_point_sources_file
+    from soxs.spatial import FillFOVModel
+
+    emin = 0.05
+    emax = 10.0
+    nbins = 10000
+    if nH is None:
+        nH = float(soxs_cfg.get("soxs", "bkgnd_nH"))
+    else:
+        nH = parse_value(nH, "1.0e22*cm**-2")
+    if absorb_model is None:
+        absorb_model = soxs_cfg.get("soxs", "bkgnd_absorb_model")
+    if frgnd_velocity is None:
+        frgnd_velocity = float(soxs_cfg.get("soxs", "frgnd_velocity"))
+    else:
+        frgnd_velocity = parse_value(frgnd_velocity, "km/s")
+    if frgnd_spec_model is None:
+        frgnd_spec_model = soxs_cfg.get("soxs", "frgnd_spec_model")
+    mylog.info("Creating galactic foreground model.")
+    spec = _make_frgnd_spectrum(
+        emin,
+        emax,
+        nbins,
+        bkgnd_nH=nH,
+        absorb_model=absorb_model,
+        frgnd_velocity=frgnd_velocity,
+        frgnd_spec_model=frgnd_spec_model,
+    )
+    sp = FillFOVModel(sky_center[0], sky_center[1], fov)
+    src = SimputSpectrum.from_models("frgnd", spec, sp, fov, 128)
+    mylog.info("Creating CXB (point sources) model.")
+    cat = make_point_sources_file(
+        filename,
+        "cxb",
+        exp_time,
+        fov,
+        sky_center,
+        absorb_model=absorb_model,
+        nH=nH,
+        area=area,
+        overwrite=overwrite,
+        input_sources=input_sources,
+        output_sources=output_sources,
+        diffuse_unresolved=diffuse_unresolved,
+        prng=prng,
+    )
+    cat.append(src, overwrite=overwrite)
+    return cat
